#ifndef _HERMIT_ARCH_RCM2_SYSTEM_H_
#define _HERMIT_ARCH_RCM2_SYSTEM_H_

#include <boost.h>

/* GPIO */
#define GPIO0_BASE_ADDR		0xe6050000
#define GPIO1_BASE_ADDR		0xe6051000
#define GPIO2_BASE_ADDR		0xe6052000
#define GPIO3_BASE_ADDR		0xe6053000
#define GPIO4_BASE_ADDR		0xe6054000
#define GPIO5_BASE_ADDR		0xe6055000
#define GPIO6_BASE_ADDR		0xe6055400
#define GPIO7_BASE_ADDR		0xe6055800

#define GPIO_IOINTSEL		0x00
/* 0:input, 1:output */
#define GPIO_INOUTSEL		0x04
/* 0:0 is outputed, 1:1 is outputed */
#define GPIO_OUTDT		0x08
/* 0:input is 0, 1:input is 1 */
#define GPIO_INDT		0x0c

/* CPG (see Table 7A-3) */
#define CPG_MSTPSR2		0xe6150040
#define CPG_SMSTPCR2		0xe6150138
#define		CPG_MSTP2_SYSDMAC0	(1<<19)
#define 	CPG_MSTP2_SCIFB2	(1<<16)
#define 	CPG_MSTP2_SCIFB1	(1<<7)
#define 	CPG_MSTP2_SCIFB0	(1<<6)
#define 	CPG_MSTP2_SCIFA0	(1<<4)
#define 	CPG_MSTP2_SCIFA1	(1<<3)
#define 	CPG_MSTP2_SCIFA2	(1<<2)

#define CPG_MSTPSR3		0xe6150048
#define CPG_SMSTPCR3	0xe615013c
#define CPG_RMSTPCR3	0xe615011c
#define 	CPG_MSTP3_MMC0		(1<<15)
#define 	CPG_MSTP3_SDHI0		(1<<14)
#define 	CPG_MSTP3_SDHI1		(1<<12)
#define 	CPG_MSTP3_SDHI2		(1<<11)

#define CPG_MSTPSR5		0xe615003c
#define CPG_SMSTPCR5		0xe6150144
#define 	CPG_MSTP5_THERMAL	(1<<22)

#define CPG_MSTPSR8		0xe61509a0
#define CPG_SMSTPCR8		0xe6150990
#define 	CPG_MSTP8_ETHER		(1<<13)

/* RESET */
#define RST_MODEMR		0xe6160060

/* SCIF */
#define SCIF0_BASE_ADDR		0xe6e60000
#define SCIF1_BASE_ADDR		0xe6e68000

#define SCIF_SMR	0x00	/* 16bit, R/W */
#define SCIF_BRR	0x04	/*  8bit, R/W */
#define SCIF_SCR	0x08	/* 16bit, R/W */
#define SCIF_FSR	0x14	/* 16bit, R/W */
#define		SCIF_FSR_ER	(1<<7)	/* Receive Error */
#define		SCIF_FSR_TEND	(1<<6)	/* Transmission END */
#define		SCIF_FSR_TDFE	(1<<5)	/* Transmission FIFO Data Empty */
#define		SCIF_FSR_BRK	(1<<4)	/* Break Detect */
#define		SCIF_FSR_FER	(1<<3)	/* Framing Error */
#define		SCIF_FSR_PER	(1<<2)	/* Parity Error */
#define		SCIF_FSR_RDF	(1<<1)	/* Receive FIFO Data Full */
#define		SCIF_FSR_DR	(1<<0)	/* Data Ready */
#define		SCIF_FSR_ERRS	(SCIF_FSR_ER | SCIF_FSR_BRK | \
				 SCIF_FSR_FER | SCIF_FSR_PER)
#define		SCIF_FSR_CLR_ERR	(SCIF_FSR_TEND | SCIF_FSR_TDFE | \
					 SCIF_FSR_BRK | SCIF_FSR_RDF | \
					 SCIF_FSR_DR)
#define SCIF_FCR	0x18	/* 16bit, R/W */

#define SCIF_DL_VALUE(bps, clk)	(clk / (bps * 16))

/* SCIFA */
#define SCIFA0_BASE_ADDR	0xe6c40000
#define SCIFA1_BASE_ADDR	0xe6c50000
#define SCIFA2_BASE_ADDR	0xe6c60000

/* SCIFB */
#define SCIFB0_BASE_ADDR	0xe6c20000
#define SCIFB1_BASE_ADDR	0xe6c30000
#define SCIFB2_BASE_ADDR	0xe6ce0000

/* HSCIF is not supported */

#define SCIFA_FDR	0x1c
#define		SCIFA_FDR_MASK	0x7f
#define 	SCIFA_FDR_FULL	0x3f
#define SCIFA_FTDR	0x20
#define SCIFA_FRDR	0x24

#define SCIFB_TFDR	0x38
#define SCIFB_RFDR	0x3c
#define		SCIFB_FDR_MASK	0x1ff
#define		SCIFB_FDR_FULL	0x0ff
#define SCIFB_FTDR	0x40
#define SCIFB_FRDR	0x60

/* I2C */
#define I2C0_BASE_ADDR		(mmu_active ? 0xe5f20000 : 0xfff20000)
#define I2C1_BASE_ADDR		0xe6c20000
#define I2C(PORT, REG)		(I2C##PORT##_BASE_ADDR + I2C_##REG)
#define I2C_ICDR		0x00
#define I2C_ICCR		0x04
#define I2C_ICSR		0x08
#define I2C_ICIC		0x0c
#define I2C_ICCL		0x10
#define I2C_ICCH		0x14
#define I2C_ICTR		0x18
#define I2C_ICRR		0x1c
#define I2C_ICTA		0x20
#define I2C_ICTB		0x24
#define I2C_ICTC		0x28
#define I2C_ICTD		0x2c
#define I2C_ICSF		0x30
#define I2C_ICSTART		0x70

/* SDHI */
#define SDHI0_BASE_ADDR		0xee100000
#define SDHI1_BASE_ADDR		0xee140000
#define SDHI2_BASE_ADDR		0xee160000
#define SDHI(PORT, REG)		(SDHI##PORT##_BASE_ADDR + SDHI_##REG)
#define SDHI_CMD		0x00
#define SDHI_PORTSEL		0x02
#define SDHI_ARG0		0x04
#define SDHI_ARG1		0x06
#define SDHI_STOP		0x08
#define SDHI_SECCNT		0x0a
#define SDHI_RSP0		0x0c
#define SDHI_RSP1		0x0e
#define SDHI_RSP2		0x10
#define SDHI_RSP3		0x12
#define SDHI_RSP4		0x14
#define SDHI_RSP5		0x16
#define SDHI_RSP6		0x18
#define SDHI_RSP7		0x1a
#define SDHI_INFO1		0x1c
#define SDHI_INFO2		0x1e
#define SDHI_INFO1_MASK		0x20
#define SDHI_INFO2_MASK		0x22
#define SDHI_CLK_CTRL		0x24
#define SDHI_SIZE		0x26
#define SDHI_OPTION		0x28
#define SDHI_ERR_STS1		0x2c
#define SDHI_ERR_STS2		0x2e
#define SDHI_BUF		0x30
#define SDHI_MODE		0x34
#define SDHI_SDIO_INFO		0x36
#define SDHI_SDIO_INFO_MASK	0x38
#define SDHI_CC_EXT_MODE	0xd8
#define SDHI_SOFT_RST		0xe0
#define SDHI_VERSION		0xe2
#define SDHI_EXT_ACC		0xe4
#define SDHI_EXT_SWAP		0xf0

/* MMCIF */
#define MMCIF_BASE_ADDR		0xee200000
#define MMC(REG)		(MMCIF_BASE_ADDR + MMC_##REG)
#define MMC_CMD_SET		0x00
#define MMC_ARG			0x08
#define MMC_ARG_CMD12		0x0c
#define MMC_CMD_CTRL		0x10
#define MMC_BLOCK_SET		0x14
#define MMC_CLK_CTRL		0x18
#define MMC_BUF_ACC		0x1c
#define MMC_RESP3		0x20
#define MMC_RESP2		0x24
#define MMC_RESP1		0x28
#define MMC_RESP0		0x2c
#define MMC_RESP_CMD12		0x30
#define MMC_DATA		0x34
#define MMC_BOOT		0x3c
#define MMC_INT			0x40
#define MMC_INT_MASK		0x44
#define MMC_HOST_STS1		0x48
#define MMC_HOST_STS2		0x4c
#define MMC_CLK_CTRL2		0x70
#define MMC_VERSION		0x7c
#define MMC_DELAY_SEL		0x80
#define MMC0_CKCR		0xe6150240

/* TMU */
#define TMU_TSTR0		0xe61e0004

#define TMU0_BASE_ADDR		0xe61e0008
#define TMU1_BASE_ADDR		0xe61e0014
#define TMU2_BASE_ADDR		0xe61e0020
#define TMU(PORT, REG)		(TMU##PORT##_BASE_ADDR + TMU_##REG)
#define TMU_TCOR		0x00
#define TMU_TCNT		0x04
#define TMU_TCR			0x08

/* CPG */
#define CPG_BASE_ADDR		0xe6150000
#define CPG(REG)		(CPG_BASE_ADDR + CPG_##REG)
#define CPG_MSTPSR		0x030
#define CPG_MSTPCR		0x130
#define REG_MSTPSR(nr)		(CPG_BASE_ADDR + CPG_MSTPSR + nr * 4)
#define REG_MSTPCR(nr)		(CPG_BASE_ADDR + CPG_MSTPCR + nr * 4)

#define cpg_set_mstpcr(nr)				\
({							\
	u32 _v = read32(REG_MSTPCR((nr) / 100));	\
	_v |= (1 << ((nr) % 100));			\
	write32(REG_MSTPCR((nr) / 100), _v);		\
})
#define cpg_clr_mstpcr(nr)				\
({							\
	u32 _v = read32(REG_MSTPCR((nr) / 100));	\
	_v &= ~(1 << ((nr) % 100));			\
	write32(REG_MSTPCR((nr) / 100), _v);		\
})

/* SYSC */
#define SYSC_BASE_ADDR		0xe6180000
#define SYSC(REG)		(SYSC_BASE_ADDR + SYSC_##REG)
#define SYSC_RESCNT		0x801c

#define BOOT_MODE_AREA0		0x00
#define BOOT_MODE_QSPI_A	0x04
#define BOOT_MODE_QSPI_B	0x08
#define BOOT_MODE_QSPI_C	0x0c

/* RWDT */
#define RWDT_BASE_ADDR		0xe6020000
#define RWTCNT0			(RWDT_BASE_ADDR + 0x00)
#define RWTCSRA0		(RWDT_BASE_ADDR + 0x04)
#define RWTCSRB0		(RWDT_BASE_ADDR + 0x08)

#endif

