/*
 * boot.S: assembly language bootstrap
 *
 * Copyright (c) 2014 Atmark Techno, Inc.
 */

#include <autoconf.h>
#include "board.h"

	.section .pre.boot
	.global _start
_start:

#if defined(CONFIG_SYSTEM_ENTRYPOINT_FLASH)
	ldr	pc, =_start_
_start_:
#endif
	/*
	 * set the cpu to SVC32 mode
	 */
	mrs	r0, cpsr
	bic	r0, r0, #0x1f
	orr	r0, r0, #0xd3
	msr	cpsr,r0

	/* Set V=0 in CP15 SCTRL register - for VBAR to point to vector */
	mrc	p15, 0, r0, c1, c0, 0	@ Read CP15 SCTRL Register
	bic	r0, #8192		@ V = 0
	mcr	p15, 0, r0, c1, c0, 0	@ Write CP15 SCTRL Register

#if defined(CONFIG_IRQ_RCM2)
	/* Set vector address in CP15 VBAR register */
	ldr	r0, =_vector
	mcr	p15, 0, r0, c12, c0, 0	@Set VBAR
#endif

	/*
	 * Invalidate L1 I/D
	 */
	mov	r0, #0			@ set up for MCR
	mcr	p15, 0, r0, c8, c7, 0	@ invalidate TLBs
	mcr	p15, 0, r0, c7, c5, 0	@ invalidate icache
	mcr	p15, 0, r0, c7, c5, 6	@ invalidate BP array
	mcr     p15, 0, r0, c7, c10, 4	@ DSB
	mcr     p15, 0, r0, c7, c5, 4	@ ISB

	/*
	 * disable MMU stuff and caches
	 */
	mrc	p15, 0, r0, c1, c0, 0
	bic	r0, r0, #0x00002000	@ clear bits 13 (--V-)
	bic	r0, r0, #0x00000007	@ clear bits 2:0 (-CAM)
	orr	r0, r0, #0x00000002	@ set bit 1 (--A-) Align
	orr	r0, r0, #0x00000800	@ set bit 11 (Z---) BTB
	bic	r0, r0, #0x00001000	@ clear bit 12 (I) I-cache
	mcr	p15, 0, r0, c1, c0, 0

	/*
	 * lowlevel-init
	 */
	mrc	p15, 0, r4, c0, c0, 5	/* mpidr */
	orr	r4, r4, r4, lsr #6
	and	r4, r4, #7		/* id 0-3 = ca15.0,1,2,3 */

	/* surpress wfe if ca15 */
	tst	r4, #4
	mrceq	p15, 0, r0, c1, c0, 1	/* actlr */
	orreq	r0, r0, #(1<<7)
	mcreq	p15, 0, r0, c1, c0, 1
	/* and set l2 latency */
	mrceq	p15, 1, r0, c9, c0, 2	/* l2ctlr */
	orreq	r0, r0, #0x00000800
	orreq	r0, r0, #0x00000003
	mcreq	p15, 1, r0, c9, c0, 2

	ldr	sp, =0xe6340000
	/* initialize system */
	blx	arch_init

goto_main:
	msr	cpsr_c, #0x52	/* MODE: IRQ */
	ldr	sp, =STACK_IRQ_START
	msr	cpsr_c, #0x13	/* MODE: SVC */
	ldr	sp, =STACK_SVC_START
	ldr	r0, =DRAM_HERMIT_START
	blx	entry /* see entry.c */

1:	b	1b
