#include <autoconf.h>
#include <hermit.h>
#include <htypes.h>
#include <arch/system.h>
#include <arch/pinmux.h>

#define read8(addr) \
	({ unsigned char __v = (*(volatile unsigned char *) (addr)); __v; })
#define read16(addr) \
	({ unsigned short __v = (*(volatile unsigned short *) (addr)); __v; })
#define read32(addr) \
	({ unsigned int __v = (*(volatile unsigned int *) (addr)); __v; })

#define write8(addr, b) \
	(void)((*(volatile unsigned char *) (addr)) = (b))
#define write16(addr, b) \
	(void)((*(volatile unsigned short *) (addr)) = (b))
#define write32(addr, b) \
	(void)((*(volatile unsigned int *) (addr)) = (b))

static void write_gp(u32 pin, gpsel gp)
{
	u32 val;

	val = read32(PIN2GPSR(pin));

	if (gp == GP_GPIO)
		val &= ~PIN2GPMASK(pin);
	else
		val |= PIN2GPMASK(pin);

	write32(PMMR, ~val);
	write32(PIN2GPSR(pin), val);
}

static void write_ip(u32 pin, ipsel ip)
{
	u32 val;

	val = read32(PIN2IPSR(pin));

	val &= ~PIN2IPMASK(pin);
	val |= (ip << PIN2IPSHIFT(pin));

	write32(PMMR, ~val);
	write32(PIN2IPSR(pin), val);
}

static void write_pu(u32 pin, pusel pu)
{
	u32 val;

	val = read32(PIN2PUPR(pin));

	if (pu == PU_DIS)
		val &= ~PIN2PUMASK(pin);
	else
		val |= PIN2PUMASK(pin);

	write32(PIN2PUPR(pin), val);
}

/*
 * (1) Procedure for changing pin function from GPIO to peripheral function
 *     1. Set the PMMR
 *     2. Set the IPSR
 *     3. Set the PMMR
 *     4. Set the GPSR to peripheral function
 * (2) Procedure for changing pin function from peripheral function to GPIO
 *     1. Set the PMMR
 *     2. Set the GPSR to GPIO
 * (3) Procedure for changing pin function from peripheral function to another peripheral function
 *     1. Set the PMMR
 *     2. Set the GPSR to GPIO
 *     3. Set the PMMR
 *     4. Set the IPSR
 *     5. Set the PMMR
 *     6. Set the GPSR to peripheral function
 */
void pinmux(u32 pin, gpsel gp, ipsel ip, pusel pu)
{
	if (IS_VALID_GP(pin))
		write_gp(pin, GP_GPIO);

	if (gp == GP_FUNC) {
		if (IS_VALID_IP(pin))
			write_ip(pin, ip);
		write_gp(pin, gp);
	}

	if (IS_VALID_PU(pin))
		write_pu(pin, pu);
}

void modsel(u16 mod, modfunc func)
{
	u32 val;

	val = read32(MOD2MOD_SEL(mod));

	val &= ~MOD2MASK(mod);
	val |= (func << MOD2SHIFT(mod));

	write32(PMMR, ~val);
	write32(MOD2MOD_SEL(mod), val);
}
